/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.fabric3.fabric.builder.resource;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.fabric3.spi.builder.BuilderConfigException;
import org.fabric3.spi.builder.BuilderException;
import org.fabric3.spi.builder.resource.ResourceContainerBuilder;
import org.fabric3.spi.builder.resource.ResourceContainerBuilderRegistry;
import org.fabric3.spi.model.physical.PhysicalResourceContainerDefinition;

/**
 * Default implementation of ResourceContainerBuilderRegistry
 *
 * @version $Rev: 155 $ $Date: 2007-06-04 08:36:11 +0100 (Mon, 04 Jun 2007) $
 */
public class ResourceContainerBuilderRegistryImpl implements ResourceContainerBuilderRegistry {
    private Map<Class<?>, ResourceContainerBuilder<?>> builders =
            new ConcurrentHashMap<Class<?>, ResourceContainerBuilder<?>>();

    public <T extends PhysicalResourceContainerDefinition> void register(Class<T> clazz,
                                                                         ResourceContainerBuilder<T> builder) {
        builders.put(clazz, builder);
    }

    @SuppressWarnings({"unchecked"})
    public <T extends PhysicalResourceContainerDefinition> void build(T definition)
            throws BuilderException {
        Class<? extends PhysicalResourceContainerDefinition> key = definition.getClass();
        ResourceContainerBuilder<T> builder = (ResourceContainerBuilder<T>) builders.get(key);
        if (builder == null) {
            throw new BuilderConfigException("Builder not found for", key.getClass().getName());
        }
        builder.build(definition);
    }
}
